using System;
using System.Collections;
using log4net;
using NHibernate;


namespace Common.Data.Hibernate {
    /// <summary>
    /// All of the functionality of this class has been factored out
    /// into other classes.
    /// 
    /// Leaving this class as simply a facade providing convenient access
    /// to the other classes.
    /// 
    /// It is now just a facade on top of:
    /// <ul>
    ///     <li><see cref="SessionHolder"/></li>
    ///     <li><see cref="NHibernateTemplate"/></li> 
    /// </ul>
    ///     
    /// </summary>
	[Obsolete("Use SessionHolder instead")]
    public class NHibernateHelper {
		private static readonly ILog _log = LogManager.GetLogger(
			System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);


		/// <summary>
		/// Creates a new NHibernate Session object if one does not already exist.
		/// When running in the context of a web application the session object is
		/// stored in HttpContext items and has 'per request' lifetime. For client apps
		/// and testing with NUnit a normal singleton is used.
		/// </summary>
		/// <returns>NHibernate Session object.</returns>
		public static ISession Session {
			get {
				return SessionHolder.Current;
			}
		}

		/// <summary>
		/// Closes any open NHibernate session if one has been used in this request.<br />
		/// This is called from the EndRequest event.
		/// </summary>
		public static void CloseSession() {
            SessionHolder.CloseCurrentSession();
		}

		/// <summary>
		/// Returns the NHibernate SessionFactory object.
		/// </summary>
		/// <returns>NHibernate SessionFactory object.</returns>
		public static ISessionFactory SessionFactory {
			get { return SessionHolder.SessionFactory; }
		}

		/// <summary>
		/// Loads the specified object.
		/// </summary>
		/// <param name="type">Type.</param>
		/// <param name="id">Id.</param>
		public static object Load(Type type, object id) {
			return SessionHolder.Current.Load(type,id);
		}

		/// <summary>
		/// Loads the specified object.
		/// </summary>
		/// <param name="type">Type.</param>
		public static IList LoadAll(Type type) {
			return SessionHolder.Template.LoadAll(type);			
		}

		/// <summary>
		/// Gets the specified object.
		/// </summary>
		/// <param name="type">Type.</param>
		/// <param name="id">Id.</param>
		public static object Get(Type type, object id) {
			return SessionHolder.Current.Get(type, id);
		}

		/// <summary> 
		/// Save object item using NHibernate. 
		/// </summary> 
		/// <param name="item">Object to save</param> 
		public static void Save(object item) {
			SessionHolder.Template.Save(item);			
		}

		/// <summary> 
		/// Save or Update object item using NHibernate. 
		/// </summary> 
		/// <param name="item">Object to save or update</param> 
		public static void SaveOrUpdate(object item) {
            SessionHolder.Template.SaveOrUpdate(item);
		}

		/// <summary> 
		/// Update object item using NHibernate. 
		/// </summary> 
		/// <param name="item">Object to update</param> 
		public static void Update(object item) {
            SessionHolder.Template.Update(item);
		}

		/// <summary>
		/// Save object item using NHibernate.
		/// </summary>
		/// <param name="item">Object to delete</param>
		public static void Delete(object item) {
            SessionHolder.Template.Delete(item);
		}

		public static IList Find(string query, object[] values, NHibernate.Type.IType[] types) {
			return SessionHolder.Current.Find(query, values, types);
		}

		public static IList Find(string query, object value1, NHibernate.Type.IType type) {
			return SessionHolder.Current.Find(query, value1, type);
		}

		public static IList Find(string query) {
			return SessionHolder.Current.Find(query);
		}
	}
}
