using System;
using System.Collections.Generic;
using SC = System.Collections;
using System.Text;
using NHibernate;

namespace Common.Data.Hibernate
{
    public class NHibernateTemplate
    {

        public NHibernateTemplate(ISession session) 
        {
            _session = session;
        }

        private ISession _session;

        public virtual ISession Session
        {
            get { return _session; }
            set { _session = value; }
        }

        public void DoInTransaction(System.Action<ISession> action)
        {
            NHibernateUtils.DoInTransaction(Session, action);
        }

        public void Save(object o)
        {
            DoInTransaction(delegate(ISession session)
            {
                session.Save(o);
            });
        }

        public void Update(object o)
        {
            DoInTransaction(delegate(ISession session)
            {
                session.Update(o);
            });
        }

        public void SaveOrUpdate(object o)
        {
            DoInTransaction(delegate(ISession session)
            {
                session.SaveOrUpdate(o);
            });
        }

        public void Delete(object o)
        {
            DoInTransaction(delegate(ISession session)
            {
                session.Delete(o);
            });
        }

        public SC.IList LoadAll(Type type)
        {
            return Session.CreateCriteria(type).List();
        }

    }

    /// <summary>
    /// Extension of HibernateTemplate that provides some basic generic convenience
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class NHibernateTemplate<T> : NHibernateTemplate
    {
        public NHibernateTemplate(ISession session) : base(session) { }

        public T Load(object id)
        {
            return Session.Load<T>(id);
        }

        public IList<T> LoadAll()
        {
            return Session.CreateCriteria(typeof(T)).List<T>();
        }

        public ICriteria CreateCriteria()
        {
            return Session.CreateCriteria(typeof(T));
        }

        public ICriteria CreateCriteria(string alias)
        {
            return Session.CreateCriteria(typeof(T),alias);
        }
    }
}
