using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using NHibernate.Type;
using NHibernate;

namespace Common.Data.Hibernate
{
    /// <summary>
    /// Simple structure for holding information that 
    /// will be passed to the hibernate query
    /// </summary>
    public class QueryParameter
    {
        public QueryParameter(string name, object value, IType type)
        {
            Name = name;
            Value = value;
            Type = type;
        }

        private string _name;

        public virtual string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        private object _value;

        public virtual object Value
        {
            get { return _value; }
            set { _value = value; }
        }

        private IType _type;

        public virtual IType Type
        {
            get { return _type; }
            set { _type = value; }
        }

    }

    public class NamedQueryParameterCollection : KeyedCollection<string,QueryParameter>
    {
        protected override void InsertItem(int index, QueryParameter item)
        {            
            // remove existing instance, if any
            string key = GetKeyForItem(item);
            if (key != null) Remove(key);

            base.InsertItem(index, item);
        }

        public void Add(string name, object value)
        {
            Add(name, value, null);
        }

        public void Add(string name, object value, IType type)
        {
            base.Add(new QueryParameter(name,value,type));
        }

        protected override string GetKeyForItem(QueryParameter item)
        {
            return item.Name;
        }

        /// <summary>
        /// Set the given parameters on the query
        /// </summary>
        public void Apply(IQuery query)
        {
            foreach (QueryParameter p in this)
            {
                if (p.Type == null)
                    query.SetParameter(p.Name, p.Value);
                else
                    query.SetParameter(
                        p.Name,
                        p.Value,
                        p.Type
                        );
            }
        }
    }
}
