using System;
using System.ComponentModel;
using System.Collections.Generic;

namespace Common.Model {
	/// <summary>
	/// Generic base class for model object that have an Id
	/// </summary>
    [Serializable]
	public class IdentifiedObject<IDType> {		

        #region Properties

        private IDType id;
		public virtual IDType Id {
			get { return this.id; }
			set { this.id = value; }
		}
        
        [Browsable(false)]
		public virtual bool IsPersisted {
			get {
                return !(EqualityComparer<IDType>.Default.Equals(id, default(IDType)));                
			}
        }

        #endregion

        #region HashCode and Equals implementations

        public override bool Equals(object obj)
        {
            if (base.Equals(obj))
                return true;
            if (obj != null && this.GetType().IsInstanceOfType(obj))
            {
                IdentifiedObject<IDType> identifiedObject = (IdentifiedObject<IDType>)obj;
                return EqualityComparer<IDType>.Default.Equals(Id, identifiedObject.Id);
            }
            return false;
        }

        public override int GetHashCode()
        {
            //TODO: if id is null, generate hashcode as if ID were the super.hashCode()
            int id;
            id = this.Id.GetHashCode();
            return id;
        }

        #endregion
    }
}
