using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace Common.Web.CustomControls
{
    public class ParameterExtender
    {

        public ParameterExtender() { }

        private string _name;

        public  string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        private string _prefix;

        [Description("Text to prepend to a string parameter value")]
        public  string Prefix
        {
            get { return _prefix; }
            set { _prefix = value; }
        }

        private string _suffix;

        [Description("Text to append to a string parameter value")]
        public  string Suffix
        {
            get { return _suffix; }
            set { _suffix = value; }
        }

        private bool _excludeIfEmpty = true;
        
        [DefaultValue(true)]
        public bool ExcludeIfEmpty
        {
            get { return _excludeIfEmpty; }
            set { _excludeIfEmpty = value; }
        }

        private string _propertyName;

        [
        DefaultValue(null), 
        Description(
            "The property that should be used when generating the WHERE clause, " + 
            "if different than the parameter name")
        ]
        public virtual string PropertyName
        {
            get { return _propertyName; }
            set { _propertyName = value; }
        }


        private string _whereCompareOperator = "=";

        [DefaultValue("=")]
        public string WhereCompareOperator
        {
            get { return _whereCompareOperator; }
            set { _whereCompareOperator = value; }
        }

        private TypeCode _typeCode = TypeCode.Empty;

        [
        Description("Shorthand alternative to Type"), 
        DefaultValue(TypeCode.Empty)
        ]
        public virtual TypeCode TypeCode
        {
            get { return _typeCode; }
            set { _typeCode = value; }
        }

        private string _type;

        [Description("The parameter type (not required)")]
        public virtual string Type
        {
            get { return _type; }
            set { _type = value; }
        }

        private string[] _parameterDependencies;

        /// <summary>
        /// See <see cref="QueryFragment.ParameterDependencies"/>
        /// </summary>
        [
        TypeConverter(typeof(System.Web.UI.WebControls.StringArrayConverter)),
        Description("Names of parameters that must be set in order for the where clause to be included")
        ]
        public virtual string[] ParameterDependencies
        {
            get { return _parameterDependencies; }
            set { _parameterDependencies = value; }
        }

    }

    public class ParameterExtenderCollection : List<ParameterExtender>
    {

        public ParameterExtender this[string name]
        {
            get
            {
                foreach (ParameterExtender pe in this)
                {
                    if (pe.Name == name) return pe;
                }
                return null;
            }
        }

    }
}
